// --------------------------------------------------------------------
// Canvas to display an image and run floodfill
// --------------------------------------------------------------------

import java.awt.{Graphics2D,Color,Dimension,Insets}
import scala.swing._
import scala.swing.event._

trait Flood {
  def flood(im: Image, p: Pixel, color: Int)
}

class Canvas(val image: Image, val flood: Flood) extends Component {
  
  focusable = true
  listenTo(mouse.clicks)
  reactions += {
    case MouseClicked(_,p,_,_,_) => mouseClicked(p.x, p.y)
  }

  // Compute (pixel size, x0, y0)
  private def gridParms: (Int, Int, Int) = {
    val d = size
    val rowWid = d.height / image.height
    val colWid = d.width / image.width
    val wid = rowWid min colWid
    val x0 = (d.width - image.width * wid)/2
    val y0 = (d.height - image.height * wid)/2
    (wid, x0, y0)
  }

  override def paintComponent(g : Graphics2D) {
    val d = size
    g.setColor(Color.white);
    g.fillRect(0,0, d.width, d.height);
    val (wid, x0, y0) = gridParms
    for (x <- 0 until image.width) {
      for (y <- 0 until image.height) {
	g.setColor(new Color(image.get(new Pixel(x, y))))
	g.fillRect(x0 + x * wid, y0 + y * wid, wid, wid)
      }
    }
  }

  def mouseClicked(xx: Int, yy: Int) {
    val (wid, x0, y0) = gridParms
    val x = (xx - x0) / wid
    val y = (yy - y0) / wid
    if (0 <= x && x < image.width && 0 <= y && y < image.height) {
      printf("Mouse clicked at (%d,%d)\n", x, y)
      flood.flood(image, new Pixel(x, y), 0xff0000)
      repaint()
    }
  }
}

// --------------------------------------------------------------------
