// --------------------------------------------------------------------
// Pixel and Image classes
// --------------------------------------------------------------------

import javax.imageio.ImageIO
import java.awt.image.BufferedImage

// Represent (x,y) coordinates
class Pixel(val x: Int, val y: Int)

class Image(fname: String) {
  private var wd: Int = 0
  private var ht: Int = 0
  private val data = readImage(fname)

  // Create image by reading from image file.
  private def readImage(filename: String): Array[Array[Int]] = {
    val f = new java.io.File(filename)
    val bi = ImageIO.read(f)
    
    wd = bi.getWidth()
    ht = bi.getHeight()
    val data = Array.ofDim[Int](wd, ht)
    for (x <- 0 until width)
      for (y <- 0 until height)
	data(x)(y) = (bi.getRGB(x, y) & 0xffffff)
    data
  }

  // Return width and height
  def width: Int = wd
  def height: Int = ht

  // Return color of the pixel.
  def get(p: Pixel): Int = data(p.x)(p.y)
  // Set color of pixel.
  def set(p: Pixel, color: Int) = { data(p.x)(p.y) = color }

  // Return array with neighbors of a pixel
  def neighbors(p: Pixel): Array[Pixel] = {
    var nb = Array.empty[Pixel]
    if (p.x > 0)
      nb = nb :+ new Pixel(p.x-1, p.y)
    if (p.y > 0)
      nb = nb :+ new Pixel(p.x, p.y-1)
    if (p.x + 1 < width) 
      nb = nb :+ new Pixel(p.x+1, p.y)
    if (p.y + 1 < height)
      nb = nb :+ new Pixel(p.x, p.y+1)
    nb
  }
}

